﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using Infragistics.Controls.Grids;
using Infragistics.Controls.Grids.Primitives;

namespace LongTextColumnPOC
{
    public class LongTextColumnContentProvider : TextColumnContentProviderBase
    {
        private const int DoubleClick = 2;

        private LongTextColumn LongTextColumn { get; set; }

        public LongTextColumnContentProvider()
        {
            TextControl.MouseDown += OnMouseDoubleClick;
            TextControl.MouseMove += OnMouseMove;
        }

        void OnMouseMove(object sender, MouseEventArgs e)
        {
            if (!(LongTextColumn.IsLongTextReadOnly && string.IsNullOrWhiteSpace(TextControl.Text)))
            {
                TextControl.Cursor = Cursors.Hand;
            }
        }

        public override FrameworkElement ResolveDisplayElement(Cell cell, Binding cellBinding)
        {
            LongTextColumn = cell.Column as LongTextColumn;
            if (LongTextColumn != null)
            {
                bool isreadonly = LongTextColumn.IsLongTextReadOnly;
                isreadonly = LongTextColumn.IsReadOnly;
            }
            //ColumnUtil.ApplyExtraStyles(TextControl, cell);
            return base.ResolveDisplayElement(cell, cellBinding);
        }

        /// <summary>
        /// Called during EnsureContent to allow the provider a chance to modify its display based on the current conditions.
        /// </summary>
        /// <param name="cell"></param>
        public override void AdjustDisplayElement(Cell cell)
        {
            base.AdjustDisplayElement(cell);
            TextControl.SetValue(TextBlock.TextProperty, cell.Value);
            SetTooltip(TextControl.Text);
            //ColumnUtil.ApplyExtraStyles(TextControl, cell);
        }

        public override object ResolveValueFromEditor(Cell cell)
        {

            return TextControl.Text;
        }

        protected override FrameworkElement ResolveEditorControl(Cell cell, object editorValue, double availableWidth, double availableHeight, Binding editorBinding)
        {
            TextControl.SetBinding(TextBox.TextProperty, editorBinding);
            return TextControl;
        }

        public void OnMouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            if (e.ClickCount == DoubleClick && e.ChangedButton == MouseButton.Left)
            {
                MessageBox.Show(LongTextColumn.IsReadOnly.ToString());

                var rowData = TextControl.DataContext;
                var propertyInfo = rowData.GetType().GetProperty(LongTextColumn.IsEditableMemberPath);
                bool isEditable = (bool)propertyInfo.GetValue(rowData);

                var window = new LongTextDialog { LongTextValue = TextControl.Text, FieldName = LongTextColumn.HeaderText, IsLongTextReadOnly = isEditable, ResizeMode = LongTextColumn.IsLongTextDialogResizable };

                if (!(LongTextColumn.IsLongTextReadOnly && string.IsNullOrWhiteSpace(TextControl.Text)))
                {
                    if (LongTextColumn.MaxLength.HasValue)
                    {
                        window.longTextBox.MaxLength = LongTextColumn.MaxLength.Value;
                    }
                    window.longTextBox.Focus();
                    window.longTextBox.Select(window.longTextBox.Text.Length, 0);
                    window.ShowDialog();

                    if (window.DialogResult.HasValue && window.DialogResult.Value)
                    {
                        TextControl.Text = window.LongTextValue;
                        SetTooltip(window.LongTextValue);
                    }
                }
            }
        }
    }
}
